@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: egg-editable-toolbar.c 2545 2007-07-03 21:40:58Z jhaitsma $
+ *  $Id: egg-editable-toolbar.c 2595 2007-07-29 16:36:00Z carlosgc $
  */
 
 #include "config.h"
@@ -1364,6 +1364,9 @@ egg_editable_toolbar_dispose (GObject *object)
       priv->visibility_paths = NULL;
     }
 
+  g_free (priv->popup_path);
+  priv->popup_path = NULL;
+
   if (priv->manager != NULL)
     {
       if (priv->visibility_id)
