@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ev-file-helpers.c 2436 2007-05-04 15:17:37Z rburton $
+ *  $Id: ev-file-helpers.c 2478 2007-05-25 16:43:19Z carlosgc $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,7 +50,7 @@ ensure_dir_exists (const char *dir)
 	if (g_file_test (dir, G_FILE_TEST_IS_DIR))
 		return TRUE;
 	
-	if (g_mkdir (dir, 488) == 0)
+	if (g_mkdir_with_parents (dir, 488) == 0)
 		return TRUE;
 
 	if (errno == EEXIST)
@@ -145,6 +145,43 @@ ev_tmp_filename (const gchar *prefix)
 	return filename;
 }
 
+/* Remove a local temp file created by evince */
+void
+ev_tmp_filename_unlink (const gchar *filename)
+{
+	const gchar *tempdir;
+	
+	if (!filename)
+		return;
+
+	tempdir = g_get_tmp_dir ();
+	if (g_ascii_strncasecmp (filename, tempdir, strlen (tempdir)) == 0) {
+		g_unlink (filename);
+	}
+}
+
+void
+ev_tmp_uri_unlink (const gchar *uri)
+{
+	GnomeVFSURI *vfs_uri;
+	gchar       *filename;
+	
+	if (!uri)
+		return;
+	
+	vfs_uri = gnome_vfs_uri_new (uri);
+	if (!gnome_vfs_uri_is_local (vfs_uri)) {
+		g_warning ("Attempting to delete non local uri: %s\n", uri);
+		gnome_vfs_uri_unref (vfs_uri);
+		return;
+	}
+	gnome_vfs_uri_unref (vfs_uri);
+
+	filename = g_filename_from_uri (uri, NULL, NULL);
+	ev_tmp_filename_unlink (filename);
+	g_free (filename);
+}
+
 gboolean
 ev_xfer_uri_simple (const char *from,
 		    const char *to,
