@@ -119,20 +119,23 @@ struct _EvJobRender
 	EvJob parent;
 
 	EvRenderContext *rc;
+	gboolean page_ready;
 	gint target_width;
 	gint target_height;
-	GdkPixbuf *pixbuf;
+	cairo_surface_t *surface;
 
 	GList *link_mapping;
 	GdkRegion *text_mapping;
 	GList *image_mapping;
+	GList *form_field_mapping;
 
-	GdkPixbuf *selection;
+	cairo_surface_t *selection;
 	GdkRegion *selection_region;
 	EvRectangle selection_points;
 	GdkColor base;
 	GdkColor text; 
 
+	gint include_forms : 1;
 	gint include_links : 1;
 	gint include_text : 1;
 	gint include_selection : 1;
@@ -142,6 +145,8 @@ struct _EvJobRender
 struct _EvJobRenderClass
 {
 	EvJobClass parent_class;
+
+	void    (* page_ready) (EvJobRender *job);
 };
 
 struct _EvJobThumbnail
@@ -194,6 +199,7 @@ struct _EvJobPrint
 	gint n_ranges;
 	EvPrintPageSet page_set;
 	gint copies;
+	gint pages_per_sheet;
 	gboolean collate;
 	gboolean reverse;
 	gdouble width;
@@ -223,6 +229,7 @@ EvJob          *ev_job_render_new         (EvDocument      *document,
 					   EvRectangle     *selection_points,
 					   GdkColor        *text,
 					   GdkColor        *base,
+					   gboolean         include_forms,
 					   gboolean         include_links,
 					   gboolean         include_images,
 					   gboolean         include_text,
@@ -258,6 +265,7 @@ EvJob          *ev_job_print_new          (EvDocument      *document,
 					   EvPrintRange    *ranges,
 					   gint             n_ranges,
 					   EvPrintPageSet   page_set,
+					   gint             pages_per_sheet,
 					   gint             copies,
 					   gdouble          collate,
 					   gdouble          reverse);
