@@ -25,6 +25,7 @@
 #include "ev-pixbuf-cache.h"
 #include "ev-page-cache.h"
 #include "ev-image.h"
+#include "ev-form-field.h"
 
 /* Information for middle clicking and moving around the doc */
 typedef struct {
@@ -73,11 +74,10 @@ typedef enum {
 } EvPresentationState;
 
 struct _EvView {
-	GtkWidget parent_instance;
+	GtkLayout layout;
 
 	EvDocument *document;
 
-	char *status;
 	char *find_status;
 	int find_result;
 	gboolean jump_to_find_result;
@@ -142,9 +142,11 @@ struct _EvView {
 	/* Image DND */
 	ImageDNDInfo image_dnd_info;
 
+#if !GTK_CHECK_VERSION (2, 11, 7)
 	/* Links */
 	GtkWidget *link_tooltip;
 	EvLink *hovered_link;
+#endif
 
 	/* Goto Popup */
 	GtkWidget *goto_window;
@@ -152,11 +154,8 @@ struct _EvView {
 };
 
 struct _EvViewClass {
-	GtkWidgetClass parent_class;
+	GtkLayoutClass parent_class;
 
-	void	(*set_scroll_adjustments) (EvView         *view,
-					   GtkAdjustment  *hadjustment,
-					   GtkAdjustment  *vadjustment);
 	void    (*binding_activated)	  (EvView         *view,
 					   EvScrollType   scroll,
 					   gboolean        horizontal);
