@@ -21,6 +21,8 @@
 #define __EV_VIEW_H__
 
 #include <gtk/gtkwidget.h>
+#include <gtk/gtklayout.h>
+#include <gtk/gtkscrolledwindow.h>
 
 #include "ev-document.h"
 #include "ev-link.h"
@@ -107,11 +109,8 @@ void		ev_view_set_zoom	  (EvView         *view,
 					   double          factor,
 					   gboolean        relative);
 double		ev_view_get_zoom	  (EvView         *view);
-void		ev_view_set_zoom_for_size (EvView         *view,
-					   int             width,
-		    			   int             height,
-					   int             vsb_width,
-					   int             hsb_height);
+void            ev_view_set_screen_dpi    (EvView         *view,
+					   gdouble         dpi);
 void		ev_view_rotate_left       (EvView         *view);
 void            ev_view_rotate_right      (EvView         *view);
 void            ev_view_set_rotation      (EvView         *view,
@@ -139,10 +138,11 @@ void	       ev_view_scroll             (EvView         *view,
 					   gboolean        horizontal);
 void	       ev_view_handle_link        (EvView         *view,
 					   EvLink         *link);
-void           ev_view_goto_dest          (EvView         *view,
-					   EvLinkDest     *dest);
 gboolean       ev_view_next_page	  (EvView         *view);
 gboolean       ev_view_previous_page	  (EvView         *view);
+gchar*         ev_view_page_label_from_dest (EvView *view, EvLinkDest *dest);
+
+void           ev_view_update_view_size (EvView *view, GtkScrolledWindow *scrolled_window);
 
 G_END_DECLS
 
